import mongoose from 'mongoose';
import Promise from 'bluebird';
import logger from './utils/logger';
import glob from 'glob';
import config from '../config/config';
import hoh8 from './sites/hoh8';
import movie from './sites/movie';
import book from './sites/book';
import bookChapter from './sites/bookChapter';
import dandanzan from './sites/dandanzan';
import proxy from './sites/proxy';
import commonTask from './sites/common';
import spiderSchedule from './schedule/spider.schedule';
mongoose.Promise = Promise;


const db = mongoose.connection;

db.on('error', function (err) {
  logger.error(err);
  process.exit(1);
});
db.once('open', async function () {
  logger.info('Connected to db.');
  // await proxy.run();
  // hoh8.run();
  // book.run();
  // movie.run();
  // bookChapter.run();
  // await dandanzan.run();
  // await commonTask.run();
  spiderSchedule.scheduleUpdateAll();
});
mongoose.connect(config.db, {promiseLibrary: Promise, useNewUrlParser: true});

let models = glob.sync(config.root + './models/*.js');
models.forEach(function (model) {
  require(model);
});

