import mongoose from 'mongoose';

const Schema = mongoose.Schema;

const Movies = new Schema({
  gid: {type: Number, required: true, index: true},
  // 电影名
  name: {type: String},
  nameAlias: {type: String},
  // 类型
  type: [{type: String}],
  // 电影评分
  score: {type: Number},
  // 电影介绍
  introduce: {type: String},
  // 语言
  language: {type: String},
  img: {type: String},
  // 导演
  daoYan: [{type: String}],
  // 主演
  zhuYan: [{type: String}],
  // 资源
  resources: [{
    _id: false,
    title: {type: String},
    link: {type: String},
  }],
  // 地区
  region: [{type: String}],
  // 年份
  year: {type: Number},
  // 真实的播放数量
  playCount: {type: Number, default: 0},
  // 显示的播放数量
  showCount: {type: Number, default: 1000},
  // 排序字段, 倒序
  sortIdx: {type: Number, default: 0},
  // jc排序字段， 倒序，查询电影时候优先此字段倒序
  sortJc: {type: Number, default: 0},
  // 是否公开
  open: {type: Boolean, default: false},
  // 购买价格
  price: {type: Number, default: 100},
  // 是否是新记录
  is_new: {type: Boolean, default: true},
  // 是否已修改
  is_modify: {type: Boolean, default: false},
  // 视频分类，movie: 电影；tv: 电视剧； show: 综艺节目
  category: {type: String},
  // 是否已删除
  deleted: {type: Boolean, default: false},
  delete_time: {type: Date},
  // 是否已发布, 客户端只会获取到published为true的游戏
  published: {type: Boolean, default: false},
}, {
  collection: 'movies',
  timestamps: true,
});
class MovieClass {
  static findByGid = function(gid) {
    return MovieModel.findOne({gid: gid, deleted: false});
  }
  static updateOne = async function (gid, record) {
    const query = {gid: gid};
    const options = {upsert: true, setDefaultsOnInsert:true};
    await MovieModel.update(query, record, options);
  }
  updateResources(arr) {
    let isModify = false;
    if (this.resources.length !== arr.length) {
      isModify = true;
    } else {
      for (let i = 0; i < this.resources.length; i ++) {
        if (this.resources[i].title !== arr[i].title
        || this.resources[i].link !== arr[i].link) {
          isModify = true;
          break;
        }
      }
    }
    this.resources = arr;
    this.is_modify = isModify;
  }
}
Movies.loadClass(MovieClass);
const MovieModel = mongoose.model('Movies', Movies);
export default MovieModel;
