import mongoose from 'mongoose';

let Schema = mongoose.Schema;


let CrawlRecordSchema = new Schema({
  // 爬取的链接地址
  url: {type: String},
  // 执行爬取任务的class
  className: {type: String},
  // 执行爬取任务的方法
  methodName: {type: String},
  // 执行爬取任务的方法的参数
  params: {type: Schema.Types.Mixed},
  lastTry: {type: Date, default: new Date()},
  tryCount: {type: Number, default: 0},
  errCount: {type: Number, default: 0},
  lastStatus: {type: Boolean}
}, {
  collection: 'crawl_record',
  timestamps: true
});

class CrawlRecordClass {
  static async updateRecord(record) {
    const query = {url: record.url};
    const options = {upsert: true, setDefaultsOnInsert:true};
    let incObj = {tryCount: 1}
    if (!record.lastStatus) {
      incObj.errCount = 1;
    }
    record['$inc'] = incObj;
    record.lastTry = new Date();
    await CrawlRecordModel.update(query, record, options);
  }
}
CrawlRecordSchema.loadClass(CrawlRecordClass);

let CrawlRecordModel = mongoose.model('CrawlRecord', CrawlRecordSchema);

export default CrawlRecordModel;
