import mongoose from 'mongoose';
import stringUtil from '../../utils/string.util';

let Schema = mongoose.Schema;


let ProxyInfoSchema = new Schema({
  //http, socket4, socket5
  type: {type: String},
  link: {type: String},
  info: {type: String},
  last_check: {type: Date},
  //相应时间
  response_time: {type: Number},
  // 1: 存活， 0: 未知，-1: 不可用
  status: {type: Number, default: 0},
  // 连续尝试次数
  try_count: {type: Number, default: 0},
  err_count: {type: Number, default: 0}
}, {
  collection: 'proxy_info',
  timestamps: true
});


let ProxyInfoModel = mongoose.model('ProxyInfo', ProxyInfoSchema);
ProxyInfoModel.updateOne = async function (link, record) {
  const query = {link: link};
  const options = {upsert: true, setDefaultsOnInsert:true};
  await ProxyInfoModel.update(query, record, options);
}

// 返回所有待检查的代理
ProxyInfoModel.needCheckList = function(all) {
  if (all) {
    return ProxyInfoModel.find({status: {$in: [0 ,1, -1]}});
  } else {
    return ProxyInfoModel.find({status: {$in: [0 ,1]}});
  }
}

// 所有可用代理
ProxyInfoModel.availableList = function() {
  return ProxyInfoModel.find({status: 1});
}

ProxyInfoModel.randomProxy = function () {
  return ProxyInfoModel.find({status: 1}).limit(1);
}

export default ProxyInfoModel;
