import netUtil from '../utils/net.util';
import SpiderCategory from '../models/SpiderCategory';
import SpiderData from '../models/SpiderData';
import BookChapter from '../models/BookChapter';
import generalQueue from '../utils/general.queue';

let Multiprogress = require("multi-progress")
let multi = new Multiprogress(process.stderr);

const generateHeader = () => {
  return {
    'User-Agent': 'quanminxiaoshuo_apple/1.0.9 CFNetwork/976 Darwin/18.2.0',
    'ver': '5.0',
    'pt': 'ios',
    'package': 'quanminxiaoshuo',
    'token': 'tc3ahpxco5ads8utnsbhxfuypkstc2oi',
    'user': '2300097'
  }
}
/**
 * 获取分类列表
 * @param {number} sex 性别 1: 男，2：女
 * */
const categoryList = async (sex) => {
  let url = `https://bc.dzjgmp.com/core/book/type.api?sex=${sex}`
  let data = await netUtil.getData(url, generateHeader());
  data = JSON.parse(data);
  for (let obj of data.data) {
    obj.sex = sex;
    let record = new SpiderCategory({
      data: obj,
      type: 'book'
    });
    await record.save();
  }
}
/**
 * 获取某分类下所有书籍列表
 * @param {number} sex sex 性别 1: 男，2：女
 * @param {number} ltype 大类
 * @param {number} page 分页 1开始
 * */
const bookList = async (sex, ltype, page) => {
  console.log(`get book list, sex: ${sex}, ltype: ${ltype}, page: ${page}`);
  let url = `https://bc.dzjgmp.com/core/book/type_all.api?sex=${sex}&ltype=${ltype}&stype=-1&status=-1&page=${page}&limit=20`;
  try {
    let data = await netUtil.getData(url, generateHeader());
    data = JSON.parse(data);
    return data;
  } catch (err) {
    console.log(err);
    return {data: {list: []}};
  }
}

/**
 * 单本书籍的详情
 * @param {string} bookId 书籍id
 * */
const bookInfo = async (bookId) => {
  console.log(`parse book: ${bookId}`);
  //书籍详情
  let url = `https://cache.dzjgmp.com/book.api?book_id=${bookId}`;
  let data = await netUtil.getData(url, generateHeader());
  data = JSON.parse(data);
  return data.data;
}
/**
 * 获取书本章节列表
 * @param {string} bookId 书本id
 * @param {string} siteId 线路id，从书本详情返回的data.sites中获取
 * */
const bookChapterList = async (bookId, siteId) => {
  console.log(`get chapter list of book: ${bookId}, siteId: ${siteId}`);
  let url = `https://cache.dzjgmp.com/book_chapter.api?book_id=${bookId}&site_id=${siteId}`;
  let data = await netUtil.getData(url, generateHeader());
  data = JSON.parse(data);
  return data;
}
/**
 * 获取单个章节的详情
 * @param {number} chapterId  章节id
 * @param {number} crawlBookId 从章节列表返回数据中的 data.crawl_book_id 获取
 * @param {number} listId 从章节列表返回数据中的 data.lists_id 获取
 * @param {string} siteId 线路id
 * */
const bookChapterInfo = async (chapterId, crawlBookId, listId, siteId) => {
  // console.log(`get chapter info of chapter: ${chapterId}, crawlBookId: ${crawlBookId}, listId: ${listId}, siteId: ${siteId}`);
  let url = `https://cache.dzjgmp.com/chapter.api?chapter_id=${chapterId}&crawl_book_id=${crawlBookId}&id=${listId}&site_id=${siteId}`;
  let data = await netUtil.getData(url, generateHeader());
  data = JSON.parse(data);
  return data;
}
// 处理单页的列表数据，保存进库
const parsePageObj = async (dataArr) => {
  for(let data of dataArr) {
    let record = new SpiderData({
      data: data,
      type: 'book'
    })
    await record.save();
  }
}
/**
 * 根据所有分类获取书籍列表信息，sex值小于参数设定值或者分类小于设定值的分类数据不作处理
 * @param {number} sex 性别信息
 * @param {number} ltype 分类
 * */
const parseAllCategory = async (sex, ltype) => {
  sex = sex || 1;
  ltype = ltype || 0;
  try {
    let cateList = await SpiderCategory.find({type: 'book'});
    for(const cate of cateList) {
      if (cate.data.sex < sex || cate.data.ltype_id < ltype) {
        continue;
      }
      let firstPage = await bookList(cate.data.sex, cate.data.ltype_id, 1);
      let totalPage = firstPage.data.last_page;
      cate.data.total = firstPage.data.total;
      cate.markModified('data');
      await cate.save();
      console.log(`parse ltype: ${cate.data.ltype_id}, totalPage: ${totalPage}`);
      await parsePageObj(firstPage.data.list);
      for(let i = 2; i <= totalPage; i ++) {
        let pageObj = await bookList(cate.data.sex, cate.data.ltype_id, i);
        await parsePageObj(pageObj.data.list);
      }
    }
  } catch (err) {
    console.log(err);
  }
}

/**
 * 处理一本书
 * 1. 获取书籍详情
 * 2. 获取章节列表
 * 3. 获取章节详情
 * */
const parseOneBook = async (book) => {
  console.log(`begin parse book: ${book.data.book_id}, name: ${book.data.name}`);
  const bookId = book.data.book_id;
  let record = await bookInfo(bookId);
  for (const key in record) {
    if ({}.hasOwnProperty.call(record, key)) {
      book.data[key] = record[key];
    }
  }
  book.status = 1;
  book.markModified('data');
  await book.save();
  if (!book.data || !book.data.sites || book.data.sites.length === 0) {
    return;
  }
  let hasChapter = false;
  // 书籍章节列表
  // 有些源无法获取到章节列表， 所以挨个获取，直到获取到数据
  if (book.data.sites && book.data.sites.length > 0) {
    for(let site of book.data.sites) {
      try {
        let chapterListData = await bookChapterList(bookId, site.site);
        if (chapterListData.code === 1) {
          hasChapter = true;
          chapterListData.book_id = bookId;
          chapterListData.site = site.site;
          const chapterExist = await BookChapter.findOne({'data.book_id': bookId, 'data.site': site.site});
          if (!chapterExist) {
            let chapterRecord = new BookChapter({
              data: chapterListData,
              type: 'book_chapter'
            });
            await chapterRecord.save();
          }
        }
      } catch (err) {
        console.log(err);
      }
    }
  }
  if (hasChapter) {
    try {
      book.status = 2;
      await book.save();
    } catch (err) {
      console.log(err);
    }
  }
  // let bar = multi.newBar('  '+book.data.name+' [:bar] :percent :etas', {
  //   complete: '=',
  //   incomplete: ' ',
  //   width: 30,
  //   total: chapterList.length
  // });
  // for (let chapter of chapterList) {
    // let obj = await bookChapterInfo(chapter.chapter_id, chapter.crawl_book_id, chapter.lists_id, siteId);
    // bar.tick(1);
    // chapter.content = obj.data.content;
  //   chapter.book_id = bookId;
  //   chapter.site = siteId;
  // }
}
const parseAllBookList = async (sex, ltype, status) => {
  let bookList = await SpiderData.find({
    'data.sex': sex,
    'data.ltype_id': ltype,
    status: status,
    type: 'book'
  });
  for(let book of bookList) {
    generalQueue.addQueue({
      run: async function () {
        await parseOneBook(book);
      }
    })
  }
}



export default {
  run: async () => {
    //step 1: 获取所有的分类
    // await categoryList(1);
    // await categoryList(2);
    // step 2: 获取所有分类下的书籍列表
    // 获取所有分类下的数据列表
    // await parseAllCategory(2, 0);
    // 获取所有数据的详情
    for(let i = 0; i < 14; i++){
      await parseAllBookList(1, i, 1);
    }
    for(let i = 0; i < 9; i++){
      await parseAllBookList(2, i, 1);
    }
  }
}
