import WebSocket from 'ws';
import logger from '../utils/logger';

function WebSocketClient() {
  this.number = 0;	// Message number
  this.autoReconnectInterval = 5 * 1000;	// ms
}

WebSocketClient.prototype.open = function (url, id) {
  this.url = url;
  this.id = id;
  this.instance = new WebSocket(this.url);
  this.instance.on('open', () => {
    this.onopen();
  });
  this.instance.on('message', (data, flags) => {
    this.number++;
    this.onmessage(data, flags, this.number);
  });
  this.instance.on('close', (e) => {
    switch (e) {
      case 1000:	// CLOSE_NORMAL
        logger.info("WebSocket: closed");
        break;
      default:	// Abnormal closure
        this.reconnect(e);
        break;
    }
    this.onclose(e);
  });
  this.instance.on('error', (e) => {
    switch (e.code) {
      case 'ECONNREFUSED':
        this.reconnect(e);
        break;
      default:
        this.onerror(e);
        break;
    }
  });
};
WebSocketClient.prototype.send = function (data, option) {
  try {
    this.instance.send(data, option);
  } catch (e) {
    this.instance.emit('error', e);
  }
};
WebSocketClient.prototype.sendTestText = function (data, option) {
  try {
    let msg = {
      msg_type: 'text',
      content: data
    };
    let msgObj = {
      type: 'm',
      bot_id: '591bf8c21d2860389057fafa',
      group: '6334778342@chatroom',
      messages: [msg]
    };
    this.instance.send(JSON.stringify(msgObj), option);
  } catch (e) {
    this.instance.emit('error', e);
  }
};
WebSocketClient.prototype.reconnect = function (e) {
  logger.info(`WebSocketClient: retry in ${this.autoReconnectInterval}ms`, e);
  this.instance.removeAllListeners();
  let that = this;
  setTimeout(function () {
    logger.info("WebSocketClient: reconnecting...");
    that.open(that.url, that.id);
  }, this.autoReconnectInterval);
};
WebSocketClient.prototype.onopen = function (e) {
  logger.info(arguments, "WebSocketClient: open");
};
WebSocketClient.prototype.onmessage = function (data, flags, number) {
  logger.info(arguments, "WebSocketClient: message");
};
WebSocketClient.prototype.onerror = function (e) {
  logger.info(arguments, "WebSocketClient: error");
};
WebSocketClient.prototype.onclose = function (e) {
  logger.info(arguments, "WebSocketClient: closed");
};


WebSocketClient.client = null;

WebSocketClient.getClient = function () {
  if (this.client === null) {
    this.client = new WebSocketClient();
  }
  return this.client;
};

export default WebSocketClient.getClient();
