import fs from 'fs-extra';
import FileStreamRotator from 'file-stream-rotator';
import bunyan from 'bunyan';
import config from '../../config/config';


let env = process.env.NODE_ENV || 'development';
let isDev = env === 'development';

const logDir = config.logs_path;
fs.existsSync(logDir) || fs.mkdirSync(logDir);

let logger = null;
let createLogger = function(appName) {
    appName = !appName ? config.app.name : appName;
    let streams = [{
        level: 'info',
        stream: FileStreamRotator.getStream({
            date_format: 'YYYYMMDD',
            filename: `${logDir}/${appName}-%DATE%.log`,
            frequency: 'daily',
            verbose: false
        })
    }];
    if (isDev) {
        streams.push({
            level: 'debug',
            stream: process.stdout
        })
    }
    return bunyan.createLogger({
        name: appName,
        serializers: bunyan.stdSerializers,
        streams: streams,
        src: false
    })
};
export default {
    info(obj, msg) {
        if (!logger) {
            logger = createLogger(global.app_name);
        }
        if (msg) {
            logger.info(obj, msg);
        } else {
            logger.info(obj);
        }
    },
    error(obj, msg) {
        if (!logger) {
            logger = createLogger(global.app_name);
        }
        if (msg) {
            logger.error(obj, msg);
        } else {
            logger.error(obj);
        }
    },
    warn(obj, msg) {
        if (!logger) {
            logger = createLogger(global.app_name);
        }
        if (msg) {
            logger.warn(obj, msg);
        } else {
            logger.warn(obj);
        }
    },
    debug(obj, msg) {
        if (!logger) {
            logger = createLogger(global.app_name);
        }
        if (msg) {
            logger.debug(obj, msg);
        } else {
            logger.debug(obj);
        }
    },
    trace(obj, msg) {
        if (!logger) {
            logger = createLogger(global.app_name);
        }
        if (msg) {
            logger.trace(obj, msg);
        } else {
            logger.trace(obj);
        }
    },
    fatal(obj, msg) {
        if (!logger) {
            logger = createLogger(global.app_name);
        }
        if (msg) {
            logger.fatal(obj, msg);
        } else {
            logger.fatal(obj);
        }
    }
}
