import request from 'request';
import Promise from 'bluebird';
import random_useragent from 'random-useragent';
import proxy from './proxy.util';
import child_process from 'child_process'


const iconv = require('iconv-lite');

const requestData = (options, encoding) => {
  return new Promise((resolve, reject) => {
    request(options, (err, response, body) => {
      if (err) {
        return reject(err);
      }
      if (response.statusCode >= 300) {
        return reject(new Error(' server response code: ' + response.statusCode + ' with url: ' + options.url));
      }
      if (encoding) {
        body = iconv.decode(body, encoding);
      }
      resolve(body);
    });
  });
}

const curlData = (url) => {
  let urlExec = `curl ${url}`
  return new Promise((resolve, reject) => {
    let child = child_process.exec(urlExec, function(err, stdout, stderr) {
      console.log(stdout);
      if (err) {
        reject && reject(err)
        return
      }
      if (stderr) {
        reject && reject(stderr)
        return
      }
      resolve && resolve(stdout)
    });
  })

}

export default {
  postForm(url, data, header, gzip) {
    header = header || {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
    }
    header['User-Agent'] = random_useragent.getRandom();
    const options = {
      method: 'POST',
      url: url,
      headers: header,
      form: data,
    };
    (gzip) && (options.gzip = true);
    return requestData(options);
  },
  getData(url, header, encoding, gzip) {
    header = header || {
      'Cache-Control': 'no-cache',
    }
    header['User-Agent'] = random_useragent.getRandom();
    const options = {
      method: 'GET',
      url: url,
      headers: header,
    };
    if (encoding) {
      options.encoding = null;
    }
    (gzip) && (options.gzip = true);
    return requestData(options, encoding);
  },
  // 通过代理来get数据
  getDataProxy(url, header, encoding, gzip) {
    header = header || {
      'Cache-Control': 'no-cache',
    }
    header['User-Agent'] = random_useragent.getRandom();
    const options = {
      method: 'GET',
      url: url,
      headers: header,
      proxy: proxy.randomProxy(),
    };
    if (encoding) {
      options.encoding = null;
    }
    (gzip) && (options.gzip = true);
    return requestData(options, encoding);
  },
  // 检查proxy是否可用
  async checkProxy(link){
    let header = {
      'Cache-Control': 'no-cache',
      'User-Agent': random_useragent.getRandom(),
      'Connection':'close',
    }
    //https://wechat-test.kingsome.cn/api/stat
    //https://www.baidu.com/favicon.ico
    const options = {
      url: 'https://wechat-test.kingsome.cn/api/stat',
      proxy: link,
      method: 'GET',
      headers: header,
      time : true,
      followRedirect: false,
      // pool: false,
    }
    return new Promise((resolve, reject) => {
      let req = request(options, (err, response, body) => {
        if (err) {
          return reject(err);
        }
        console.log('status code:', response.statusCode, 'body:', body);
        if (response.statusCode === 200 && body === '1') {
          console.log('Request time in ms', response.elapsedTime);
          resolve(response.elapsedTime);
        } else {
          reject(new Error(' server response code: ' + response.statusCode + ' with url: ' + options.url));
        }
      });
      setTimeout(() => {
        console.log('check proxy timeout', link);
        try {
          req.abort(new Error('timeout'));
        } catch (err) {
          console.log(err);
          reject()
        }
        reject()
      }, 5000);
    })

  }
}
