import crypto from "crypto";
import format from 'biguint-format';

export default {
  /**
   * 从url中截取文件名
   * @param {string} url
   * @return {string} filename
   * */
  getFileNameFromUrl(url) {
    return url.substring(url.lastIndexOf('/')+1)
  },

  removeHtml(content, replceEnter) {
    if (replceEnter) {
      return content.replace(/<.+?>/g, '')
        .replace(/\r\n/g, '<br/>')
        .replace(/\s/g, '')
        .replace(/(<br\/>)+/g, '<br/>')
        .replace(/^<br\/>/, '')
        .replace(/<br\/>$/, '');
    } else {
      return content.replace(/<.+?>/g, '').replace(/\s/g, '');
    }
  },
  /**
   * 根据正则查找内容
   * @param {string} content
   * @param {RegExp} re 正则表达式， 例如：/\/.+?\/(\d+?)\.html/
   * @return {string} 匹配到的内容， 没找到则返回''
   * */
  findByReg(content, re) {
    const contents = content.match(re);
    let result = '';
    if (contents) {
      if (contents.length > 1) result = contents[1];
    }
    return result;
  },
  // randomNum(minNum, maxNum) {
  //   return parseInt(Math.random()*(maxNum-minNum+1)+minNum, 10);
  // },
  randomNum(minNum, maxNum) {
    let x = format(crypto.randomBytes(4), 'dec');
    return parseInt(x / Math.pow(2, 4 * 8) * (maxNum + 1 - minNum) + minNum);
  },
  getContentByReg(str, reg) {
    const contents = str.match(reg);
    if (contents && contents.length > 1) {
      return contents[1]
    } else {
      return '';
    }
  }
}
